#!/usr/bin/env python3

import pyshark
import struct
import sys
import binascii

def hexDump(s):
  h = " ".join(map("{:02x}".format, s))
  t = ""
  for i in range(len(s)):
    c = s[i]
    if c>=0x20 and c<0x7f:
      t += chr(c)
    else:
      t += "."
  return( h + " | " + t)

def isWriteCommand(p):
  if not isDataPacket(p):
    return False
  data = getData(p)
  return 'W' == chr(data[0])

def isDataPacket(p):
  return ("host" == p.usb.src) and ("USB.CAPDATA" in p)

def getData(p):
  if not isDataPacket(p): 
    return None
  return binascii.a2b_hex(p["USB.CAPDATA_RAW"].value)


def dumpWriteCommand(p, nextaddr):
  if not isWriteCommand(p):
    return
  data = getData(p)
  res = struct.unpack(">cIB16sBB", data)
  addr = res[1]
  if nextaddr != addr:
    print((8+3+16*3+16+2)*"-")
  print("{:08X} : {}".format(addr, hexDump(res[3])))
  return addr+16

cap = pyshark.FileCapture(sys.argv[1], include_raw=True, use_json=True)
nextaddr = 0
for p in cap:
  if isWriteCommand(p):
    nextaddr = dumpWriteCommand(p, nextaddr)



from abc import ABCMeta, abstractmethod

class StreamHandler(metaclass=ABCMeta):
  def __init__(self):
    pass 
  
  @abstractmethod
  def handleFromHost(self, data:bytearray):
    pass 

  @abstractmethod
  def handleToHost(self, data:bytearray):
    pass 




#!/usr/bin/env python3

""" 
THis module implements some basic packet handler, that can filter and process packets captured by
wireshark. 
"""

from abc import ABCMeta, abstractmethod
from pyshark import FileCapture


class PacketHandler(metaclass=ABCMeta):
  """ Abstract base class for all packet handlers. """

  def __init__(self):
    self._handler = []

  def attach(self, handler):
    self._handler.append(handler)
    return self

  @staticmethod
  def isFromHost(p):
    return "host" == p.usb.src

  @staticmethod
  def isToHost(p):
    return "host" == p.usb.dest

  def handle(self, packet):
    for handler in self._handler:
      handler.handle(packet)

  def process(self, filename:str):
    for packet in FileCapture(filename, include_raw=True, use_json=True):
      self.handle(packet)
    


def hexDump(s: bytes, prefix: str = "", addr: int = 0) -> str:
  """ Utility function to hex-dump binary data. """
  N = len(s)
  Nb = N//16
  if (N%16): Nb += 1
  res = ""
  for j in range(Nb):
    a,b = j*16, min((j+1)*16,N)

    h = " ".join(map("{:02x}".format, s[a:b]))
    h += "   "*(16-(b-a))
    t = ""
    for i in range(a,b):
      c = s[i]
      if c>=0x20 and c<0x7f:
        t += chr(c)
      else:
        t += "."
    res += (prefix + "{:08X} ".format(addr+16*j) + h + " | " + t + "\n")
  return res[:-1]



#!/usr/bin/env python3
#
# Emulate anytone d878uv radio to customer programming software. 
# Send intercepted data stream over network to server script for further investigation.
#
# This script connects to a virtual com port COM26 which is connected via a virtual
# null modem cable to the virtual com port COM18 which is used by the programming software.
# This virtual ports and cable can be provided by the COM0COM tool.
#
# Linux users can use
# socat -d -d pty,raw,echo=0,b4000000 pty,raw,echo=0,b4000000
# for emulating a virtual null modem cable.

import serial
import time
import sys
import struct 


# config
filebase = 'codeplug' 
filecount = 0
comport = 'COM6' # connected to COM18 with com0com. use COM18 in CPS


def hexDump(s):
  h = " ".join(map("{:02x}".format, s))
  t = ""
  for i in range(len(s)):
    c = s[i]
    if c>=0x20 and c<0x7f:
      t += chr(c)
    else:
      t += "."
  return( h + " | " + t)

# parameters?
if len(sys.argv) == 2:
   comport = sys.argv[1]
elif len(sys.argv) >= 3:
   print("Usage: " + sys.argv[0] + ' [comport]')
   exit()



# open serial port
serialPort = None

try:
   print("Trying comport " + comport)
   serialPort = serial.Serial(port = comport, baudrate=4000000, bytesize=8, timeout=1, stopbits=serial.STOPBITS_ONE) # 115200 921600 4000000
except (err):
   print('ERR: Could not open port ' + comport)
   print("Usage: " + sys.argv[0] + ' servername [comport]: ' + err)
   exit()


out = None 
nextaddr = None 

# wait for data

try:
   
   while 1:

      command = ''

      command = serialPort.read()
      while serialPort.in_waiting > 0:
         command += serialPort.read()

      # respond to command on com port
      if ( len(command) == 0 ):
         pass

      elif ( command == b'PROGRAM'):
         print("Program session requested.")
         resp = b'QX\x06'
         serialPort.write(resp)
         filename = "{}_{:04}.hex".format(filebase, filecount)
         out = open(filename, "w")
         nextaddr = None 
         
      elif ( command == b'\x02' ):
         print("Device info requested.")
         resp = b'ID868UVE\x00V102\x00\x00\x06'
         serialPort.write(resp)

      elif ( command == b'R\x02\xfa\x00\x20\x10' ):
         print("Read special memory request.")
         resp = b'W\x02\xfa\x00\x20\x10\xff\xff\xff\xff\x00\x00\xff\xff\x00\x00\x00\x00\x00\x00\x00\x00\x26\x06'
         serialPort.write(resp)
         
      elif ( command[0:4] == b'R\x02\xfa\x00' and command[5] == 16 ):
         # 0x02fa00..
         print("Read local information.")
         
         resp = b'W\x02\xfa\x00' + bytes([command[4]]) + b'\x10'
         
         if ( command[4] == 0x00 ):
            resp += b'\x00\x00\x00\x00\x01\x01\x01\x00\x00\x01\x01\x20\x20\x20\x20\xff'
                                           
         elif ( command[4] == 0x10 ): # Radio Type
            resp += b'\x44\x38\x37\x38\x55\x56\x00\x01\x00\xff\xff\xff\xff\xff\xff\xff'

         elif ( command[4] == 0x30 ): # Serial Number
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0x40 ): # Production Date
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0x50 ): # Manucfacture Code
            resp += b'\x31\x32\x33\x34\x35\x36\x37\x38\xff\xff\xff\xff\xff\xff\xff\xff'
         elif ( command[4] == 0x60 ): # Maintained Date
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0x70 ): # Dealer Code
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0x80 ): # Stock Date
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0x90 ): # Sell Date
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0xa0 ): # Seller
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'

         elif ( command[4] == 0xb0 ): # Maintained Description
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0xc0 ): 
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0xd0 ): 
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0xe0 ): 
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
         elif ( command[4] == 0xf0 ): 
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'

         else:
            resp += b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'

         resp = resp + bytes( [sum(resp[1:]) & 0xff] ) + b'\x06'
         #print(resp.hex())
         serialPort.write(resp)

      elif ( command[0] == ord('W') ) :
         resp = b'\x06' # just ack
         serialPort.write(resp)
         res = struct.unpack(">cIB16sBB", command)
         addr = res[1]
         if nextaddr != addr:
            out.write((8+3+16*3+16+2)*"-" + "\n")
         out.write("{:08X} : {}\n".format(addr, hexDump(res[3])))
         nextaddr = addr+16
         
      elif ( command == b'END' ):
         print("End session.")
         resp = b'\x06' # just ack
         serialPort.write(resp)
         out.close()
         filecount += 1

      elif ( command == b'UPDATE' ):
         # for firmware update the device has to be switched on while pressing PF3 (blue button on top) and PTT keys
         print("Start Firmware Update. Only useful if device is in update receiving mode. (Switch on while pressing PF3 (blue button on top) and PTT keys)")
         resp = b'\x06' # just ack
         serialPort.write(resp)

      elif ( command == b'\x18' ):
         print("Firmware Update Send Complete. Switch device on while pressing PF2 (top left side) and PTT keys to start installer.")
         resp = b'\x06' # just ack
         serialPort.write(resp)

      elif ( command[0] == 0x01 ):
         print("Firmware data.")
         resp = b'\x06' # just ack
         serialPort.write(resp)
      else:
         #print("> " + str(command))
         pass



finally:
   print('QRT')
   serialPort.close()




from abc import ABCMeta, abstractmethod


class Datagram(metaclass=ABCMeta):
  """ Base class of all data grams. """

  def __init__(self):
    pass 

  @abstractmethod
  def format(self, prefix=""):
    pass


class DatagramHandler(metaclass=ABCMeta):

  def __init__(self):
    pass 

  @abstractmethod
  def handleFromHost(self, dgram: Datagram):
    pass 

  @abstractmethod 
  def handleToHost(self, dgram: Datagram):
    pass 


class RawDatagramDump(DatagramHandler):
  def __init__(self):
    super().__init__()

  def handleFromHost(self, dgram:Datagram):
    print(dgram.format("< "))

  def handleToHost(self, dgram:Datagram):
    print(dgram.format("> "))



from packethandler import PacketHandler
from streamhandler import StreamHandler
from binascii import a2b_hex


class CDCACMFilter(PacketHandler):
  
  USB_URB_INTERRUPT = 0x01
  USB_URB_BULK      = 0x03

  def __init__(self):
    super().__init__()
    pass 

  def attach(self, handler:StreamHandler):
    self._handler.append(handler)
    return self

  def handle(self, packet):
    if (int(packet.usb.transfer_type,base=16) != CDCACMFilter.USB_URB_BULK) or ("USB.CAPDATA" not in packet):
      return
    for handler in self._handler:
      if PacketHandler.isFromHost(packet):
        handler.handleFromHost(a2b_hex(packet["USB.CAPDATA_RAW"].value))
      else:
        handler.handleToHost(a2b_hex(packet["USB.CAPDATA_RAW"].value))



#!/usr/bin/env python3

import pyshark
import sys
import binascii
from abc import ABCMeta

dev_addr = 1

if len(sys.argv) != 2:
    print("Usage: extract.py PCAPNG_FILE")


def hexDump(s: bytes, prefix: str = "", addr: int = 0) -> str:
  """ Utility function to hex-dump binary data. """
  N = len(s)
  Nb = N//16
  if (N%16): Nb += 1
  res = ""
  for j in range(Nb):
    a,b = j*16, min((j+1)*16,N)

    h = " ".join(map("{:02x}".format, s[a:b]))
    h += "   "*(16-(b-a))
    t = ""
    for i in range(a,b):
      c = s[i]
      if c>=0x20 and c<0x7f:
        t += chr(c)
      else:
        t += "."

  res += (prefix + "{:08X} ".format(addr+16*j) + h + " | " + t + "\n")
  return res[:-1]

def isFromHost(p):
    return ("host" == p.usb.src) and ((None==dev_addr) or (dev_addr == int(p.usb.device_address))) 

def isToHost(p):
    return (("host" == p.usb.dest) and ((None==dev_addr) or (dev_addr == int(p.usb.device_address))))

def isDataPacket(p):
  return ("USB.CAPDATA" in p)

def getData(p):
  if not isDataPacket(p): 
    return None
  return binascii.a2b_hex(p["USB.CAPDATA_RAW"].value)


cap = pyshark.FileCapture(sys.argv[1], include_raw=True, use_json=True)
nextaddr = 0
for p in cap:
  if isDataPacket(p):
    if isFromHost(p):
      print(hexDump(getData(p), "< "))
    else:
      print(hexDump(getData(p), "> "))
      #print(p)
        



from packethandler import PacketHandler

class DeviceFilter(PacketHandler):
  """ Filters by device address. """

  def __init__(self, device: int = None):
    """ :param device: Specifies the USB device address of radio. """
    super().__init__()
    self._source = device

  def handle(self, packet):
    if ((None == self._source) or (int(packet.usb.device_address) == self._source)):
      super().handle(packet)



from devicefilter import DeviceFilter
from streamhandler import StreamHandler
from cdcacmfilter import CDCACMFilter
from rawstreamdump import RawStreamDump
from datagram import Datagram, RawDatagramDump, DatagramHandler
from struct import unpack
from utilities import hexDump
from binascii import crc_hqx

def crc16_simple_sum8(data:bytearray):
  return sum(data)&0xffff

def crc16_simple_sum16(data:bytearray):
  s = 0
  for i in range(len(data)//2):
    s += unpack(">H", data[2*i:(2*(i+1))])[0]
  return s&0xffff

def crc16_simple_xor16(data:bytearray):
  s = 0
  for i in range(len(data)//2):
    s ^= unpack(">H", data[2*i:(2*(i+1))])[0]
  return s

def crc_summary(data:bytearray):
  print("CRC16-HQX  : {:04X}".format(crc_hqx(data, 0)))
  print("CRC16-SUM8 : {:04X}".format(crc16_simple_sum8(data)))
  print("CRC16-SUM16: {:04X}".format(crc16_simple_sum16(data)))
  print("CRC16-XOR16: {:04X}".format(crc16_simple_xor16(data)))


class AuctusA6Datagram(Datagram):
  def __init__(self, command, parameters, crc):
    self._isRequest = (0 == (command & 0x8000))
    self._command = (command & 0x7fff)
    self._parameters = parameters 
    self._crc = crc 

  def isRequest(self):
    return self._isRequest

  def isResponse(self):
    return not self.isRequest()

  def command(self):
    return self._command 

  def parameters(self):
    return self._parameters

  def format(self, prefix=""):
    res = prefix
    res += "type={}, command={:04X}, CRC={:02X}".format(
      ("Req" if self._isRequest else "Resp"), self._command, self._crc)
    if len(self._parameters):
      res += "\n"+hexDump(self._parameters, prefix=" "*len(prefix)+" |")
    return res 

  def __repr__(self):
    return self.format()
    

class AuctusA6Handler(StreamHandler):
  MODE_PASS     = 0x00
  MODE_DATAGRAM = 0x01

  def __init__(self):
    super().__init__()
    self._tobuffer = bytearray()
    self._frombuffer = bytearray()
    self._mode = AuctusA6Handler.MODE_DATAGRAM
    self._dataLeft = None
    self._handler = []

  def attach(self, handler):
    self._handler.append(handler)
    return self 

  def handleToHost(self, data:bytearray):
    self._tobuffer.extend(data)
    if AuctusA6Handler().MODE_DATAGRAM == self._mode: 
      while AuctusA6Handler().hasDatagram(self._tobuffer):
        dgram = AuctusA6Handler().popDatagram(self._tobuffer)
        self._processToHost(dgram)
        for handler in self._handler:
          if isinstance(handler, DatagramHandler):
            handler.handleToHost(dgram)
    else:
      self._dataLeft -= len(data)
      if 0 == self._dataLeft:
        self._mode = AuctusA6Handler.MODE_DATAGRAM
        for handler in self._handler:
          if isinstance(handler, StreamHandler):
            handler.handleToHost(self._tobuffer)
        crc_summary(self._tobuffer)
        self._tobuffer.clear()

  def handleFromHost(self, data:bytearray):
    self._frombuffer.extend(data)
    if AuctusA6Handler().MODE_DATAGRAM == self._mode: 
      while AuctusA6Handler().hasDatagram(self._frombuffer):
        dgram = AuctusA6Handler().popDatagram(self._frombuffer)
        self._processFromHost(dgram)
        for handler in self._handler:
          if isinstance(handler, DatagramHandler):
            handler.handleFromHost(dgram)
    else:
      self._dataLeft -= len(data)
      if 0 == self._dataLeft:
        self._mode = AuctusA6Handler.MODE_DATAGRAM
        for handler in self._handler:
          if isinstance(handler, StreamHandler):
            handler.handleFromHost(self._frombuffer)
        crc_summary(self._frombuffer)
        self._frombuffer.clear()

  @staticmethod 
  def hasDatagram(buffer: bytearray):
    if not buffer.startswith(b'\xaa'): return False 
    if len(buffer) < 6: return False
    length, command = unpack(">xBH", buffer[:4])
    return len(buffer) >= length

  @staticmethod
  def popDatagram(buffer: bytearray):
    length, command = unpack(">xBH", buffer[:4])
    length, command, params, crc = unpack(">xBH{}sBx".format(length-6), buffer[:length])
    del buffer[:length]
    return AuctusA6Datagram(command, params, crc)

  def _processFromHost(self, dgram:AuctusA6Datagram):
    if dgram.isRequest() and (0x0102 == dgram.command()):
      self._dataLeft, baudRate = unpack(">xxIxxI", dgram.parameters())
    elif dgram.isRequest() and (0x0101 == dgram.command()):
      self._mode = AuctusA6Handler.MODE_PASS

  def _processToHost(self, dgram:AuctusA6Datagram):
    if dgram.isResponse() and (0x0102 == dgram.command()):
      self._mode = AuctusA6Handler.MODE_PASS
    elif dgram.isResponse() and (0x0100 == dgram.command()):
      self._dataLeft, = unpack(">xI10x", dgram.parameters())

if "__main__" == __name__:
  import sys
  stream = DeviceFilter(14).attach(
        CDCACMFilter().attach(
          AuctusA6Handler()
            .attach(RawDatagramDump())
            .attach(RawStreamDump())))
  stream.process(sys.argv[1])



from utilities import hexDump
from streamhandler import StreamHandler


class RawStreamDump(StreamHandler):
  def __init__(self):
    super().__init__()
    self._toAddr = 0
    self._fromAddr = 0

  def handleFromHost(self, data):
    print(hexDump(data, "< ", self._fromAddr))
    self._fromAddr+=len(data)

  def handleToHost(self, data):
    print(hexDump(data, "> ", self._toAddr))
    self._toAddr+=len(data)



#!/usr/bin/env python3

import pyshark
import struct
import sys
import binascii

device = "1.255"
device_in = "{}.2".format(device)
device_out = "{}.1".format(device)

def hexDump(s: bytes, prefix:str="", addr:int=0, compact:bool=False) -> str:
  """ Utility function to hex-dump binary data. """
  N = len(s)
  Nb = N//16
  if (N%16): Nb += 1
  res = ""
  last_line = None
  compressing = False
  for j in range(Nb):
    a,b = j*16, min((j+1)*16,N)
    line = s[a:b]
    if (not compact) or (last_line != line):
      h = " ".join(map("{:02x}".format, line))
      h += "   "*(16-(b-a))
      t = ""
      for i in range(a,b):
        c = s[i]
        if c>=0x20 and c<0x7f:
          t += chr(c)
        else:
          t += "."
      res += (prefix + "{:08X} ".format(addr+16*j) + h + " | " + t + "\n")
      last_line = line
      compressing = False
    elif not compressing: 
      res += (prefix + "*\n")
      compressing = True 
  return res[:-1]

def isDataPacket(p):
  return ("USB.CAPDATA" in p)

def isFromHost(p):
  return (("host" == p.usb.src) and (device_in == p.usb.dst))

def isFromDevice(p):
  return (("host" == p.usb.dst) and (device_out == p.usb.src))

def getData(p):
  if not isDataPacket(p): 
    return None
  return binascii.a2b_hex(p["USB.CAPDATA_RAW"].value)


class Packet:
  def __init__(self, data:bytes):
    self._flag, self._cmd, self._sub, self._pcrc, size = struct.unpack("<xBBBHH", data[:8])
    self._payload = bytes()
    if size: self._payload = data[8:(8+size)]
    self._crc = 0xffff
    for i in range(len(data)//2):
      v = struct.unpack("<H", data[(2*i):(2*(i+1))])[0]
      if v > self._crc: self._crc += 0xffff;
      self._crc -= v    
    if len(data)%2:
      v = struct.unpack("B", data[-1:])[0]
      if v > self._crc: self._crc +=0xffff;
      self._crc -= v;
    

  def __repr__(self):
    if 0 == len(self._payload):
      return "Packet flag={:02x}, cmd={:02x}, sub={:02x}, crc={:04x}.".format(self._flag, self._cmd, self._sub, self._crc)
    return "Packet flag={:02x}, cmd={:02x}, sub={:02x}, crc={:04x}\n{}".format(self._flag, self._cmd, self._sub, self._crc, hexDump(self._payload, " "))

cap = pyshark.FileCapture(sys.argv[1], include_raw=True, use_json=True)
nextaddr = 0
for p in cap:
  if isDataPacket(p):
    print(Packet(getData(p)))



#!/usr/bin/env python3

import pyshark
import struct
import sys
import binascii

dev_addr = 6


def hexDump(s):
  h = " ".join(map("{:02x}".format, s))
  t = ""
  for i in range(len(s)):
    c = s[i]
    if c>=0x20 and c<0x7f:
      t += chr(c)
    else:
      t += "."
  return( h + " | " + t)

def isFromHost(p):
    return ("host" == p.usb.src) and ((None==dev_addr) or (dev_addr == int(p.usb.device_address))) 

def isToHost(p):
    return ("host" == p.usb.dest) 

def isDataPacket(p):
  return ("USB.CAPDATA" in p)

def getData(p):
  if not isDataPacket(p): 
    return None
  return binascii.a2b_hex(p["USB.CAPDATA_RAW"].value)


cap = pyshark.FileCapture(sys.argv[1], include_raw=True, use_json=True)
nextaddr = 0
for p in cap:
  if isDataPacket(p):
    dump = hexDump(getData(p))
    if isFromHost(p):
      print("> " + dump)
    else:
      print("< " + dump)
        



#!/usr/bin/env python3 

import struct 
import sys 


def hexDump(s: bytes, prefix="", addr=0, compact=False) -> str:
  """ Utility function to hex-dump binary data. """
  N = len(s)
  Nb = N//16
  if (N%16): Nb += 1
  res = ""
  last_line = None
  compressing = False
  for j in range(Nb):
    a,b = j*16, min((j+1)*16,N)
    line = s[a:b]
    if (not compact) or (last_line != line):
      h = " ".join(map("{:02x}".format, line))
      h += "   "*(16-(b-a))
      t = ""
      for i in range(a,b):
        c = s[i]
        if c>=0x20 and c<0x7f:
          t += chr(c)
        else:
          t += "."
      res += (prefix + "{:08X} ".format(addr+16*j) + h + " | " + t + "\n")
      last_line = line
      compressing = False
    elif not compressing: 
      res += (prefix + "*\n")
      compressing = True 
  return res[:-1]


if 2 > len(sys.argv):
  print("USAGE: extract.py CODEPLUG.rdt")
rdt_filename = sys.argv[1]

# list of sections: (file offset, size, destination address)
sections = [
  (0x002225, 0x03e000, 0x002000),
  (0x040230, 0x090000, 0x110000),
]

# Read all
data = open(rdt_filename, "br").read()
for section in sections:
  offset, length, addr = section
  print(hexDump(data[offset:(offset+length)], "", addr, True))





#!/usr/bin/env python3


from json import dump
from re import S
import pyshark
import struct
import sys
import binascii

device = None # "1.13.0"

def hexDump(s: bytes, prefix="", addr=0) -> str:
  """ Utility function to hex-dump binary data. """
  N = len(s)
  Nb = N//16
  if (N%16): Nb += 1
  res = ""
  for j in range(Nb):
    a,b = j*16, min((j+1)*16,N)

    h = " ".join(map("{:02x}".format, s[a:b]))
    h += "   "*(16-(b-a))
    t = ""
    for i in range(a,b):
      c = s[i]
      if c>=0x20 and c<0x7f:
        t += chr(c)
      else:
        t += "."

    res += (prefix + "{:08X} ".format(addr+16*j) + h + " | " + t + "\n")
  return res[:-1]

def isFromHost(p):
  b = ("host" == p.usb.src)
  if device:
    b = b and (device == p.usb.dst)
  return b

def isFromDevice(p):
  b = ("host" == p.usb.dst)
  if device:
    b = b and (device == p.usb.src)
  return b

def isRequest(p):
  return isFromHost(p) and ("SETUP DATA" == p.highest_layer) and (p.layers[-1].has_field("data_fragment"))

def isResponse(p):
  return isFromDevice(p) and ("USB.CAPDATA" in p) 

def getData(p):
  if isRequest(p):
    return binascii.a2b_hex(p.layers[-1].data_fragment_raw[0])
  elif isResponse(p): 
    return binascii.a2b_hex(p["USB.CAPDATA_RAW"].value)
  return None


class RawPayload:
  def __init__(self, payload):
    self._payload = payload

  def __len__(self):
    return len(self._payload)

  def dump(self, prefix="", addr=0):
    return hexDump(self._payload, prefix, addr)
  

class Package:
  def __init__(self, payload):
    self._type, self._length = struct.unpack("<HH", payload[:4])
    payload = payload[4:(4+self._length)]
    if (0x52 == payload[0]) and len(payload)>=4:
      self._payload = ReadOperation(payload)
    elif (0x57 == payload[0]) and len(payload)>=4:
      self._payload = WriteOperation(payload)
    elif 0x41 == payload[0]:
      self._payload = ACK(payload)
    else:
      self._payload = RawPayload(payload)

  def type(self):
    return self._type

  def payload(self):
    return self._payload

  def dump(self, prefix=""):
    s = ""
    if 1 == self._type:
      s += prefix + "Request, len=0x{0:02X}".format(self._length) + "\n"
    elif 3 == self._type:
      s += prefix + "Response, len=0x{0:02X}".format(self._length) + "\n"
    s += self._payload.dump(prefix+"   | ")
    return s


class ReadOperation:
  def __init__(self, payload):
    self._addr, self._length = struct.unpack(">HB", payload[1:4])
    self._payload = None
    if 4 < len(payload):
      self._payload = RawPayload(payload[4:])

  def dump(self, prefix=""):
    s = prefix + "Read addr=0x{0:04X}, len=0x{1:02X}".format(self._addr, self._length)
    if None != self._payload:
      s += "\n" + self._payload.dump(prefix + "   | ")
    return s 

class WriteOperation:
  def __init__(self, payload):
    self._addr, self._length = struct.unpack(">HB", payload[1:4])
    self._payload = None
    if 4 < len(payload):
      self._payload = RawPayload(payload[4:])

  def payload(self):
    return self._payload
  def addr(self):
    return self._addr

  def dump(self, prefix=""):
    s = prefix + "Write addr=0x{0:04X}, len=0x{1:02X}".format(self._addr, self._length)
    if None != self._payload:
      s += "\n" + self._payload.dump(prefix + "   | ")
    return s 


class ACK: 
  def __init__(self, payload):
    pass 

  def dump(self, prefix=""):
    return prefix+"ACK"

if 2 == len(sys.argv):
  cap = pyshark.FileCapture(sys.argv[1], include_raw=True, use_json=True)
  for p in cap:
    if isRequest(p) and len(getData(p))>=4:
      P = Package(getData(p))
      print(P.dump(" > "))
      print("")
    elif isResponse(p) and len(getData(p))>=4:
      P = Package(getData(p))
      print(P.dump(" < "))
      print("")
    elif isRequest(p): 
      print(hexDump(getData(p), " > "))
    elif isResponse(p):
      print(hexDump(getData(p), " < "))

elif (3 == len(sys.argv)) and ("write" == sys.argv[1]):
  print("Dump written memory from file {0}:".format(sys.argv[2]))
  cap = pyshark.FileCapture(sys.argv[2], include_raw=True, use_json=True)
  addr = 0
  for p in cap:
    if isRequest(p) and len(getData(p))>=4:
      P = Package(getData(p))
      if isinstance(P.payload(), WriteOperation):
        W = P.payload()
        if (addr != W.addr()):
          print("")
          print ("-"*80)
          print("")
          addr = W.addr()
        print(W.payload().dump(" ", addr))
        addr += len(W.payload())
      else:
        print("")
        print(P.dump(" > "))
        print("")

else:
  print("oops")